﻿using System;
using System.IO;
using Android.App;
using Android.OS;
using Android.Support.V7.App;
using Android.Widget;
using Android.Graphics;

using Syncfusion.Pdf.Parsing;
using Syncfusion.Pdf.Interactive;
using Syncfusion.SfPdfViewer.Android;

namespace PDFTest
{
    [Activity(Label = "@string/app_name", Theme = "@style/AppTheme.NoActionBar", MainLauncher = true)]
    public class MainActivity : AppCompatActivity
    {
        private MemoryStream docStream = null;
        private SfPdfViewer pdfView = null;
        private ImageView imgView = null;

        protected override void OnCreate(Bundle savedInstanceState)
        {
            base.OnCreate(savedInstanceState);
            Xamarin.Essentials.Platform.Init(this, savedInstanceState);
            SetContentView(Resource.Layout.activity_main);

            Button btnLoad = FindViewById<Button>(Resource.Id.btnLoad);
            btnLoad.Click += BtnLoad_Click;

            Button btnSaveToImg = FindViewById<Button>(Resource.Id.btnSaveToImg);
            btnSaveToImg.Click += BtnSaveToImg_Click;

            Button btnTextAnnot = FindViewById<Button>(Resource.Id.btnTextAnnot);
            btnTextAnnot.Click += BtnTextAnnot_Click;
            
            imgView = FindViewById<ImageView>(Resource.Id.imageView);
            imgView.Visibility = Android.Views.ViewStates.Gone;

            pdfView = FindViewById<SfPdfViewer>(Resource.Id.vPdfViewer);
            pdfView.BookmarkPaneVisible = false;
            pdfView.BookmarkNavigationEnabled = false;


            pdfView.EnableDocumentLinkAnnotation = false;
            pdfView.IsTextSearchEnabled = false;
            pdfView.Toolbar.Enabled = false;

            pdfView.FreeTextAnnotationAdded += PdfView_FreeTextAnnotationAdded;

        }


        private void BtnLoad_Click(object sender, EventArgs e)
        {
            if (docStream != null)
            {
                docStream.Dispose();
                docStream = null;
            }
            imgView.Visibility = Android.Views.ViewStates.Gone;
            pdfView.Visibility = Android.Views.ViewStates.Visible;

            using (StreamReader reader = new StreamReader(Assets.Open("FreeTextAnnotation.pdf")))
            {
                docStream = new MemoryStream();
                reader.BaseStream.CopyTo(docStream);
            }

            // Bug 1. => I already get response that it is scheduled to fix upcoming 2019 vol 4.
            // https://www.syncfusion.com/feedback/10375/modified-freetextannotation-content-is-not-updated-in-resultant-pdf-with-korean
            // Before Viewer.LoadDocument(), I want to change contents of existing FreeTextAnnotations to Korean letter.
            PdfLoadedDocument document = new PdfLoadedDocument(docStream);
            for (int i = 0; i < document.Pages.Count; i++)
            {
                foreach (PdfLoadedAnnotation Annot in document.Pages[i].Annotations)
                {
                    if (Annot is PdfLoadedFreeTextAnnotation)
                    {
                        PdfLoadedFreeTextAnnotation TextAnnot = Annot as PdfLoadedFreeTextAnnotation;
                        if (String.IsNullOrWhiteSpace(TextAnnot.Text) == false)
                        {
                            // if the new content is in Korean, the modification is not reflected to resultant pdf file.. after v17.2.0.46
                            // but Syncfusion.Xamarin.Pdf v17.2.0.41 or lower works well.
                            //string szNewContent = "텍스트 편집";  // In Korean, changed Content is not saved to resultant stream
                            string szNewContent = "in English"; // In English, changed Content is saved well to resultant stream
                            TextAnnot.MarkUpText = Annot.Text = szNewContent;   // Change text content                          
                        }
                    }
                }
            }

            //Save the document to the stream
            document.Save(docStream);
            //document.Close(true);

            // Show It
            pdfView.LoadDocument(docStream);
        }

        private void BtnSaveToImg_Click(object sender, EventArgs e)
        {
            if (docStream == null)
                return;

            pdfView.Visibility = Android.Views.ViewStates.Gone;
            imgView.Visibility = Android.Views.ViewStates.Visible;
            
            // Bug 2. Exporting as Image cannot save annotations and form fields
            // Show exported image -> Annotations not exported to image. only original text contents exported.
            byte[] bImgBytes = null;
            using (Stream imgStream = pdfView.ExportAsImage(0))
            {
                imgStream.Position = 0;
                bImgBytes = new byte[imgStream.Length];
                imgStream.Read(bImgBytes, 0, (int)imgStream.Length);
            }
            if (bImgBytes != null)
                imgView.SetImageBitmap(BitmapFactory.DecodeByteArray(bImgBytes, 0, bImgBytes.Length));
        }


        private void BtnTextAnnot_Click(object sender, EventArgs e)
        {
            if (docStream == null)
                return;

            imgView.Visibility = Android.Views.ViewStates.Gone;
            pdfView.Visibility = Android.Views.ViewStates.Visible;

            // Bug 3. Adding new Free Text Annotation makes null referenced exception
            // Text input dialog is appeared well.
            // After typing text, Clicking "OK" button to save text, It makes null reference excetpion.
            // https://help.syncfusion.com/xamarin-android/sfpdfviewer/freetext-annotation
            pdfView.AnnotationMode = AnnotationMode.FreeText;
        }


        private void PdfView_FreeTextAnnotationAdded(object sender, FreeTextAnnotationAddedEventArgs args)
        {
            Toast.MakeText(this, "FreeTextAnnotationAdded : " + args.Text, ToastLength.Short).Show();
        }
    }
}

